import { TestBed } from '@angular/core/testing';
import { HttpClientTestingModule, HttpTestingController } from '@angular/common/http/testing';

import { IUnitType } from '../unit-type.model';
import { sampleWithRequiredData, sampleWithNewData, sampleWithPartialData, sampleWithFullData } from '../unit-type.test-samples';

import { UnitTypeService } from './unit-type.service';

const requireRestSample: IUnitType = {
  ...sampleWithRequiredData,
};

describe('UnitType Service', () => {
  let service: UnitTypeService;
  let httpMock: HttpTestingController;
  let expectedResult: IUnitType | IUnitType[] | boolean | null;

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule],
    });
    expectedResult = null;
    service = TestBed.inject(UnitTypeService);
    httpMock = TestBed.inject(HttpTestingController);
  });

  describe('Service methods', () => {
    it('should find an element', () => {
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.find(123).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'GET' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should create a UnitType', () => {
      const unitType = { ...sampleWithNewData };
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.create(unitType).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'POST' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should update a UnitType', () => {
      const unitType = { ...sampleWithRequiredData };
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.update(unitType).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'PUT' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should partial update a UnitType', () => {
      const patchObject = { ...sampleWithPartialData };
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.partialUpdate(patchObject).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'PATCH' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should return a list of UnitType', () => {
      const returnedFromService = { ...requireRestSample };

      const expected = { ...sampleWithRequiredData };

      service.query().subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'GET' });
      req.flush([returnedFromService]);
      httpMock.verify();
      expect(expectedResult).toMatchObject([expected]);
    });

    it('should delete a UnitType', () => {
      const expected = true;

      service.delete(123).subscribe(resp => (expectedResult = resp.ok));

      const req = httpMock.expectOne({ method: 'DELETE' });
      req.flush({ status: 200 });
      expect(expectedResult).toBe(expected);
    });

    describe('addUnitTypeToCollectionIfMissing', () => {
      it('should add a UnitType to an empty array', () => {
        const unitType: IUnitType = sampleWithRequiredData;
        expectedResult = service.addUnitTypeToCollectionIfMissing([], unitType);
        expect(expectedResult).toHaveLength(1);
        expect(expectedResult).toContain(unitType);
      });

      it('should not add a UnitType to an array that contains it', () => {
        const unitType: IUnitType = sampleWithRequiredData;
        const unitTypeCollection: IUnitType[] = [
          {
            ...unitType,
          },
          sampleWithPartialData,
        ];
        expectedResult = service.addUnitTypeToCollectionIfMissing(unitTypeCollection, unitType);
        expect(expectedResult).toHaveLength(2);
      });

      it("should add a UnitType to an array that doesn't contain it", () => {
        const unitType: IUnitType = sampleWithRequiredData;
        const unitTypeCollection: IUnitType[] = [sampleWithPartialData];
        expectedResult = service.addUnitTypeToCollectionIfMissing(unitTypeCollection, unitType);
        expect(expectedResult).toHaveLength(2);
        expect(expectedResult).toContain(unitType);
      });

      it('should add only unique UnitType to an array', () => {
        const unitTypeArray: IUnitType[] = [sampleWithRequiredData, sampleWithPartialData, sampleWithFullData];
        const unitTypeCollection: IUnitType[] = [sampleWithRequiredData];
        expectedResult = service.addUnitTypeToCollectionIfMissing(unitTypeCollection, ...unitTypeArray);
        expect(expectedResult).toHaveLength(3);
      });

      it('should accept varargs', () => {
        const unitType: IUnitType = sampleWithRequiredData;
        const unitType2: IUnitType = sampleWithPartialData;
        expectedResult = service.addUnitTypeToCollectionIfMissing([], unitType, unitType2);
        expect(expectedResult).toHaveLength(2);
        expect(expectedResult).toContain(unitType);
        expect(expectedResult).toContain(unitType2);
      });

      it('should accept null and undefined values', () => {
        const unitType: IUnitType = sampleWithRequiredData;
        expectedResult = service.addUnitTypeToCollectionIfMissing([], null, unitType, undefined);
        expect(expectedResult).toHaveLength(1);
        expect(expectedResult).toContain(unitType);
      });

      it('should return initial array if no UnitType is added', () => {
        const unitTypeCollection: IUnitType[] = [sampleWithRequiredData];
        expectedResult = service.addUnitTypeToCollectionIfMissing(unitTypeCollection, undefined, null);
        expect(expectedResult).toEqual(unitTypeCollection);
      });
    });

    describe('compareUnitType', () => {
      it('Should return true if both entities are null', () => {
        const entity1 = null;
        const entity2 = null;

        const compareResult = service.compareUnitType(entity1, entity2);

        expect(compareResult).toEqual(true);
      });

      it('Should return false if one entity is null', () => {
        const entity1 = { id: 123 };
        const entity2 = null;

        const compareResult1 = service.compareUnitType(entity1, entity2);
        const compareResult2 = service.compareUnitType(entity2, entity1);

        expect(compareResult1).toEqual(false);
        expect(compareResult2).toEqual(false);
      });

      it('Should return false if primaryKey differs', () => {
        const entity1 = { id: 123 };
        const entity2 = { id: 456 };

        const compareResult1 = service.compareUnitType(entity1, entity2);
        const compareResult2 = service.compareUnitType(entity2, entity1);

        expect(compareResult1).toEqual(false);
        expect(compareResult2).toEqual(false);
      });

      it('Should return false if primaryKey matches', () => {
        const entity1 = { id: 123 };
        const entity2 = { id: 123 };

        const compareResult1 = service.compareUnitType(entity1, entity2);
        const compareResult2 = service.compareUnitType(entity2, entity1);

        expect(compareResult1).toEqual(true);
        expect(compareResult2).toEqual(true);
      });
    });
  });

  afterEach(() => {
    httpMock.verify();
  });
});
